/* SystemTap probe support for GDB.

   Copyright (C) 2011 Free Software Foundation, Inc.

   This file is part of GDB.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */

#if !defined (STAP_PROBE_H)
#define STAP_PROBE_H 1

#include "vec.h"

struct stap_args_info;
struct axs_value;
struct linespec_result;

/* Main structure which holds information about a SystemTap probe.  */

struct stap_probe
{
  /* The provider of this probe.  */
  const char *provider;

  /* The name of the probe.  */
  const char *name;

  /* The address where the probe is inserted.  */
  CORE_ADDR address;

  /* The address of the probe's semaphore, or 0 if this probe does not
     have an associated semaphore.  */
  CORE_ADDR sem_addr;

  /* Probe's arguments.  Users should generally not examine this, but
     should instead extract information about the arguments using the
     methods provided in sym_probe_fns.  */
  const char *args;

  /* Probe's arguments after parsing.  This is an opaque structure that
     will hold information about the arguments pointed by ARGS.  */
  struct stap_args_info *parsed_args;
};

typedef const struct stap_probe *stap_probe_p;
DEF_VEC_P (stap_probe_p);

/* A helper for linespec that decodes a stap probe specification.  It
   returns a symtabs_and_lines object and updates *ARGPTR or throws an
   error.  */

extern struct symtabs_and_lines parse_stap_probe (char **argptr,
						  struct linespec_result *canon);

/* Search OBJFILE for a probe with the given PROVIDER and NAME.
   Return a VEC of all probes that were found.  If no matching probe
   is found, return NULL.  The caller must free the VEC.  */

extern VEC (stap_probe_p) *find_probes_in_objfile (struct objfile *objfile,
						   const char *provider,
						   const char *name);

/* Given a PC, find an associated SystemTap probe.  If a probe is
   found, set *OBJFILE_OUT to the probe's objfile, and return the
   probe.  If no probe is found, return NULL.  */

extern const struct stap_probe *find_probe_by_pc (CORE_ADDR pc,
						  struct objfile **objfile_out);

/* Given PROBE, returns the number of arguments present in that probe's
   argument string.  */

extern int stap_get_probe_argument_count (const struct stap_probe *probe);

/* Given PARSED_ARGS, frees the space allocated to hold information about
   the probe's parsed arguments.  */

extern void stap_free_parsed_args (struct stap_args_info *parsed_args);

/* Evaluates the probe's argument N, returning a value corresponding
   to it.  */

extern struct value *stap_evaluate_probe_argument (struct objfile *objfile,
						   const struct stap_probe *probe,
						   struct frame_info *frame,
						   int n);

/* Compile the probe's argument N to agent expression.  */

struct agent_expr;
extern void stap_compile_to_ax (struct objfile *objfile,
				const struct stap_probe *probe,
				struct agent_expr *expr,
				struct axs_value *value,
				int n);

/* A convenience function that finds a probe at the PC in FRAME and
   evaluates argument N.  If there is no probe at that location, or if
   the probe does not have enough arguments, this returns NULL.  */

extern struct value *stap_safe_evaluate_at_pc (struct frame_info *frame,
					       int n);

#endif /* !defined (STAP_PROBE_H) */
